
/**
 * Module dependencies.
 */

var os = require('os');

/**
 * Parse the given user-agent `str`.
 *
 * @param {String} str
 * @return {Object}
 * @api public
 */

exports.parse = function(str){
  var parts = str.split(/\s+/);
  var obj = { name: parts.shift() };
  parts = parts.join(' ').split(') (');

  parts.forEach(function(part){
    var p = part.split(/: */);
    var key = p.shift().replace('(', '');
    var val = p.shift().replace(')', '');
    obj[key] = val;
  });

  return obj;
};

/**
 * Stringify user-agent `name` and optional `obj`.
 *
 * @param {String} name
 * @param {Object} [obj]
 * @return {String}
 * @api public
 */

exports.stringify = function(name, obj){
  if (!name) throw new Error('name required');
  obj = obj || {};

  obj.node = process.version;
  obj.os = os.platform();
  obj.release = os.release();
  obj.hostname = os.hostname();
  
  var keys = Object.keys(obj);
  var ua = keys.map(function(key){
    return '(' + key + ': ' + obj[key] + ')';
  });

  return name + ' ' + ua.join(' ');
};