
var tree = require('../');

function find(name, pkg) {
  return pkg.deps.filter(function(dep){
    return dep.name == name;
  }).shift();
}

describe('.nextModuleDir(dir)', function(){
  it('should return the next node_modules dir', function(){
    var dir = 'test/fixtures/node_modules/jade/node_modules/commander/node_modules/mocha/node_modules';
    var after = 'test/fixtures/node_modules/jade/node_modules/commander/node_modules';
    tree.nextModuleDir(dir).should.equal(after);

    dir = after;
    after = 'test/fixtures/node_modules/jade/node_modules';
    tree.nextModuleDir(dir).should.equal(after);

    dir = after;
    var after = 'test/fixtures/node_modules';
    tree.nextModuleDir(dir).should.equal(after);
  })
})

describe('.dirs(dir)', function(){
  it('should return the directories to try', function(){
    var dir = 'test/fixtures/node_modules/jade/node_modules/commander/node_modules/mocha/node_modules';
    var dirs = tree.dirs(dir);
    dirs[0].should.equal('test/fixtures/modules');
    dirs[1].should.equal('test/fixtures/support');
    dirs[2].should.equal('test/fixtures/node_modules/jade/node_modules/commander/node_modules/mocha/node_modules');
    dirs[3].should.equal('test/fixtures/node_modules/jade/node_modules/commander/node_modules');
    dirs[4].should.equal('test/fixtures/node_modules/jade/node_modules');
    dirs[5].should.equal('test/fixtures/node_modules');
  })
})

describe('tree(file)', function(){
  describe('when deps are met', function(){
    it('should return a package tree', function(){
      var pkg = tree('node_modules/mocha/package.json');
      pkg.name.should.equal('mocha');
      pkg.deps[0].name.should.equal('commander');
      pkg.deps[1].name.should.equal('growl');

      var jade = find('jade', pkg);
      jade.deps[0].name.should.equal('commander');
      jade.deps[1].name.should.equal('mkdirp');
    })

    it('should support NODE_PATH', function(){
      var pkg = tree('test/fixtures/node_modules/ferrets/package.json');
      pkg.name.should.equal('ferrets');
      pkg.deps[0].name.should.equal('tobi');
      pkg.deps[0].deps[0].name.should.equal('jane');
      pkg.deps[1].name.should.equal('loki');
    })
  })

  describe('when deps are not met', function(){
    it('should error', function(){
      +function(){
        var pkg = tree('test/fixtures/node_modules/foo/package.json');
      }.should.throw('Failed to resolve package "foo"\'s dep "bar"');
    })
  })
})